@@ -87,7 +87,11 @@ class Device:
 
     def read_status(self):
         if self.status_addr is None:
-            return StatusStruct(PLCStatus.IDLE << 12)
+            if self.target_addr is None:
+                return StatusStruct(PLCStatus.IDLE << 12)
+            if self.read_value() == self.read_target():
+                return StatusStruct(PLCStatus.IDLE << 12)
+            return StatusStruct(PLCStatus.BUSY << 12)
         if self.status_size == 2:
             return StatusStruct(self.io.read_u16(self.status_addr))
         return StatusStruct(self.io.read_u32(self.status_addr) >> 16)
@@ -359,7 +363,7 @@ class ParamInterface:
     def update_param_sm(self):
         self.param_sm(self.io.read_u16(self.pctrl_addr))
 
-    def _wait_sm_available(self):
+    def wait_sm_available(self):
         self.update_param_sm()
         timesout = time.time() + self.param_timeout
         while not self.param_sm.available:
@@ -393,9 +397,11 @@ class ParamInterface:
                                   int(self.param_sm), int(value))
         else:
             if self.value_size == 4:
-                self.io.write_u16_f32(self.pctrl_addr, int(self.param_sm), value)
+                self.io.write_u16_f32(self.pctrl_addr,
+                                      int(self.param_sm), value)
             else:
-                self.io.write_fmt(self.pctrl_addr, 'Hd', int(self.param_sm), value)
+                self.io.write_fmt(self.pctrl_addr, 'Hd',
+                                  int(self.param_sm), value)
 
     def set_param_cmd(self, cmd, sub, idx):
         self.param_sm.CMD = cmd
@@ -418,27 +424,27 @@ class ParamInterface:
     def list_params(self):
         return self.param_map.init_keys()
 
-    def get_param(self, name):
+    def get_param(self, name, subdev=0):
         idx = self.param_map.get(name)
         if idx:
-            if self._wait_sm_available():
-                self.set_param_cmd(ParamCMDs.DO_READ, self.subdev, idx)
+            if self.wait_sm_available():
+                self.set_param_cmd(ParamCMDs.DO_READ, subdev, idx)
                 # now wait until setting parameter is finished and
                 # return read-back-value
-                if self._wait_sm_available():
+                if self.wait_sm_available():
                     return self.param_sm.CMD, self.get_param_value(idx)
             return ParamCMDs.ERR_RETRY, None
         return ParamCMDs.ERR_NO_IDX, None
 
-    def set_param(self, name, value):
+    def set_param(self, name, value, subdev=0):
         idx = self.param_map.get(name)
         if idx:
-            if self._wait_sm_available():
+            if self.wait_sm_available():
                 self.set_param_value_and_cmd(ParamCMDs.DO_WRITE,
-                                             self.subdev, idx, value)
+                                             subdev, idx, value)
                 # now wait until setting parameter is finished and
                 # return read-back-value
-                if self._wait_sm_available():
+                if self.wait_sm_available():
                     return self.param_sm.CMD, self.get_param_value(idx)
             return ParamCMDs.ERR_RETRY, None
         return ParamCMDs.ERR_NO_IDX, None
@@ -446,23 +452,23 @@ class ParamInterface:
     def list_funcs(self):
         return self.func_map.init_keys()
 
-    def exec_func(self, name, value=None):
+    def exec_func(self, name, value=None, subdev=0):
         idx = self.func_map.get(name)
         if idx:
             self.update_param_sm()
             if self.param_sm.CMD == ParamCMDs.BUSY and \
-               self.param_sm.SUB == self.subdev and \
+               self.param_sm.SUB == subdev and \
                self.param_sm.IDX == idx:
                 self.set_param_value(idx, value)
-                if self._wait_sm_available():
+                if self.wait_sm_available():
                     return self.param_sm.CMD, self.get_param_value(idx)
                 return self.param_sm.CMD, self.get_param_value(idx)  # still BUSY
-            if self._wait_sm_available():
+            if self.wait_sm_available():
                 self.set_param_value_and_cmd(ParamCMDs.DO_WRITE,
-                                             self.subdev, idx, value)
+                                             subdev, idx, value or 0)
                 # now wait until setting parameter is finished and return
                 # read-back-value
-                if self._wait_sm_available():
+                if self.wait_sm_available():
                     return self.param_sm.CMD, self.get_param_value(idx)
                 return self.param_sm.CMD, self.get_param_value(idx)  # still BUSY
             return ParamCMDs.ERR_RETRY, None
