@@ -72,7 +72,7 @@ class Scanner:
 
     def scan_2015_02(self):
         indexer = self.io.indexer
-        next_addr = indexer.addr + indexer.size
+        next_addr = indexer.indexer_addr + indexer.indexer_size
 
         for devnum in range(1, 256):
             info = indexer.query_infostruct(devnum)
