@@ -24,6 +24,7 @@
 
 import collections
 import struct
+from itertools import islice
 
 from zapf.simulator.util import NumProxy
 
@@ -39,6 +40,22 @@ class Memory:
         self.dyn_addr = 0x40000
         self.allocations = []
         self.allocations_sorted = False
+        self.allocs_by_size = {}
+
+    def sort_allocations(self):
+        self.allocs_by_size.clear()
+        self.allocations.sort(key=lambda a: a[:2])
+        # Allocations are sorted by size first, then by address - this lets us
+        # find the "innermost" allocation for a given byte range.  But in order
+        # to speed up this search, we cache the index where to start the search
+        # for any given size.
+        last = 0
+        for (i, (osize, _, _)) in enumerate(self.allocations):
+            if osize > last:
+                for nsize in range(last + 1, osize + 1):
+                    self.allocs_by_size[nsize] = i
+                last = osize
+        self.allocations_sorted = True
 
     def new(self, size):
         addr = self.dyn_addr
@@ -55,12 +72,15 @@ class Memory:
         if there are at least size bytes left.
         """
         if not self.allocations_sorted:
-            self.allocations.sort()
-            # print(self.allocations)
-            self.allocations_sorted = True
-        for (osize, oaddr, obj) in self.allocations:
-            if oaddr <= addr and addr + size <= oaddr + osize:
-                return (obj, addr - oaddr)
+            self.sort_allocations()
+        # Find the smallest allocation that encompasses the whole given range.
+        # If the size is not in "alloc_size_start", the requested size is
+        # bigger than any single object, which is not allowed.
+        if size in self.allocs_by_size:
+            allocs = islice(self.allocations, self.allocs_by_size[size], None)
+            for (osize, oaddr, obj) in allocs:
+                if oaddr <= addr and addr + size <= oaddr + osize:
+                    return (obj, addr - oaddr)
         raise RuntimeError('no value or addressing across value boundary')
 
     def read(self, addr, size):
@@ -339,7 +359,9 @@ def array(innertype, imin, imax):
 
 
 class Var:
-    """Represents a field in a struct."""
+    """Represents a field in a struct (which includes the "variable struct"
+    automatically generated for a function decorated with @program).
+    """
 
     def __init__(self, dtype, default=None, *, at=None):
         self.dtype = dtype
@@ -396,7 +418,8 @@ class Struct(Value, metaclass=StructMeta):
         for (name, var) in self.VARS:
             self.__dict__[var] = var.dtype.alloc(
                 getattr(value, name, pvars.pop(name, var.default)),
-                at=var.at or (addr + self.OFFSET[name] if addr is not None else None))
+                at=var.at or (addr + self.OFFSET[name]
+                              if addr is not None else None))
         if pvars:
             raise RuntimeError('unknown variable in struct: %s' % pvars)
 
@@ -418,27 +441,20 @@ class Struct(Value, metaclass=StructMeta):
                               ', '.join('%s => %r' % x for x in items))
 
     def mem_read(self):
-        return b''.join(self.__dict__[var].mem_read() for (_, var) in self.VARS)
+        return b''.join(self.__dict__[var].mem_read()
+                        for (_, var) in self.VARS)
 
     def mem_write(self, offset, data):
-        skip = 0
-        for (name, _) in self.VARS:
-            if self.OFFSET[name] == offset:
-                break
-            skip += 1
-        else:
-            raise RuntimeError(f'partial struct {self.__class__.__name__} '
-                               f'element write')
-        for (i, (name, var)) in enumerate(self.VARS):
-            if i < skip:
+        for (name, var) in self.VARS:
+            var_skip = offset - self.OFFSET[name]
+            var_size = var.dtype.sizeof()
+            # skip elements before the first offset
+            if var_skip >= var_size:
                 continue
-            size = var.dtype.sizeof()
-            if len(data) < size:
-                raise RuntimeError(f'partial struct {self.__class__.__name__} '
-                                   f'element write '
-                                   f'(size {size}, datalen {len(data)}')
-            d, data = data[:size], data[size:]
-            self.__dict__[var].mem_write(0, d)
+            write_size = min(var_size - var_skip, len(data))
+            d, data = data[:write_size], data[write_size:]
+            self.__dict__[var].mem_write(var_skip, d)
+            offset += write_size
             if not data:
                 break
 
